import { Component, Input } from "@angular/core";
import { FormBuilder, FormGroup, Validators } from "@angular/forms";
import { NgbActiveModal } from "@ng-bootstrap/ng-bootstrap";
import { BuildingService } from "../../../service/monitoring-api.service";
import { LoginService } from "../../../service/login.service";

@Component({
  selector: "app-add-edit-master-room",
  templateUrl: "./add-edit-master-room.component.html",
  styleUrls: ["./add-edit-master-room.component.css"],
})
export class AddEditMasterRoomComponent {
  @Input() dataRow: any;
  @Input() mode: any;
  myForm: FormGroup;

  data: any;
  dataStatus: any[];
  dataBuilding: any[];
  filteredDataStatus: any[];
  filteredDataBuilding: any[];
  dataMasterStatus: any;
  labelRoom: string = "";

  constructor(
    public activeModal: NgbActiveModal,
    private fb: FormBuilder,
    private monitoringApiService: BuildingService,
    private authService: LoginService
  ) {}

  ngOnInit() {
    this.authService.startTokenCheck();
    this.authService.startTrackingActivity();
    this.createForm();
    this.listDataStatus();
    this.listDataBuilding();
    console.log(this.dataRow);
    if (this.mode === "add") {
      this.labelRoom = "Add New Row"
    } else {
      this.labelRoom = "Edit Row"
    }
    if (this.dataRow) {
      this.editForm();
    }
  }

  createForm() {
    this.myForm = this.fb.group({
      name: ["", Validators.required],
      statusId: ["", Validators.required],
      buildingId: ["", Validators.required],
      description: ["", Validators.required],
    });
  }

  editForm() {
    this.myForm = this.fb.group({
      id: this.dataRow.id,
      roomId: this.dataRow.roomId,
      name: [this.dataRow.roomEntity.name, Validators.required],
      statusId: [this.dataRow.statusId, Validators.required],
      buildingId: [this.dataRow.buildingId, Validators.required],
      description: [this.dataRow.roomEntity.description, Validators.required],
    });
  }

  listDataStatus() {
    this.monitoringApiService.getMasterData().subscribe((res) => {
      this.data = res.results.data;
      this.dataMasterStatus = res.results.data.find(
        (item) => item.name === "master_status"
      );
      this.dataStatus = this.dataMasterStatus.headerDetailParam;
      this.filteredDataStatus = this.dataStatus.filter(
        (item) => item.statusName.toLowerCase() === "aktif"
      );
    });
  }

  listDataBuilding() {
    this.monitoringApiService.getMasterBuildingData().subscribe((res) => {
      this.dataBuilding = res.results.data;
      this.filteredDataBuilding = this.dataBuilding
    });
  }

  addRow() {
    if (this.myForm.valid) {
      this.activeModal.close(this.myForm.value);
    } else {
      this.markFormGroupTouched(this.myForm);
    }
  }

  markFormGroupTouched(formGroup: FormGroup) {
    (Object as any).values(formGroup.controls).forEach((control) => {
      control.markAsTouched();
      if (control.controls) {
        this.markFormGroupTouched(control);
      }
    });
  }
}
